/* AutoIK.ls  Copyright(C) 2000 NewTek, Inc.
2.24.00 Jason Linhart with help from Bob Hood and Allen Hastings

Automatically set up an IK chain by adding the GOAL null object and
changing the pitch angles of the items in the chain to be controlled 
by IK.

User Requirements: Select 2 items.  The beginning and end of the
chain.

2.22.00 Start.
2.23.00 Modified to allow selection from either end of the chain.
2.24.00 Updated to select using itemIDs instead of names.

*/

@version 2.0
@warnings
@script generic

//Globals

done = FALSE;
foundSecond = FALSE;
selectionCount = 0;

mymatch = true;
myHtoggle = false;
myPtoggle = true;
myBtoggle = false;
myIKontop = true;


//Begin

generic
{
    // Test to make sure 2 items are selected.

    mymatch = recall("MGmymatch",true);
    myHtoggle = recall("MGmyHtoggle",false);
    myPtoggle = recall("MGmyPtoggle",true);
    myBtoggle = recall("MGmyBtoggle",false);
    myIKontop = recall("MGmyIKontop",true);


    scene = getfirstitem(SCENE);

    selectedItem = scene.firstSelect();
    while(selectedItem)
    {
        selectionCount++;

        selectedItem = scene.nextSelect();
    }

    if (selectionCount != 2)
    {
        info("WARNING: 2 items must be selected");
        return;
    }

//---------------------------------------------------------------------------
    reqbegin("AutoIK by Jason Linhart (additions Michael Green)");
    reqsize(370,160);
    c1 = ctlcheckbox("Match Goal Orientation",mymatch);
    c2 = ctlcheckbox("IK on Heading",myHtoggle);
    c3 = ctlcheckbox("IK on Pitch",myPtoggle);
    c4 = ctlcheckbox("IK on Bank",myBtoggle);
    c5 = ctlcheckbox("IK on Heading just for root",myIKontop);
    return if !reqpost();

    mymatch = getvalue(c1);
    myHtoggle = getvalue(c2);
    myPtoggle = getvalue(c3);
    myBtoggle = getvalue(c4);
    myIKontop = getvalue(c5);

    reqend();

//-------------------------------------------------------------------------------    
    

    // Get info from the selected items.

    scene = getfirstitem(SCENE);
    first = scene.firstSelect();
    rootGenus = first.genus;
    second = scene.nextSelect();

    // Determine root and pointer.

    if(first.parent)
    {
        nextItem = first.parent;
        while(nextItem && (nextItem.genus == rootGenus))
        {
            if(nextItem.id == second.id)
                foundSecond = TRUE;

            nextItem = nextItem.parent;
        }

        if(foundSecond)
        {
                rootItem = second;
                pointerItem = first;
        }
        else
        {
                rootItem = first;
                pointerItem = second;
  
        }
        
    }
    else
    {
        rootItem = first;
        pointerItem = second;
    }

    // Create Goal.

    position = pointerItem.getWorldPosition(0.0);
    rotation = pointerItem.getWorldRotation(0.0);

    goalName = pointerItem.name+"_GOAL";
    AddNull(goalName);

    // Move to last object in the list (the Null we just added!)
    // to get the itemID.  Selecting by name is not assured due
    // to the fact that L[6] could auto-increment the name if there
    // is a duplicate.

    lastItem = getfirstitem(MESH);
    while(lastItem)
    {
        previousItem = lastItem;        
        lastItem = lastItem.next();
    }

    goalItem = previousItem;
    SelectItem(goalItem.id);
    Position(position.x, position.y, position.z);
    if (mymatch)
        Rotation(rotation.x, rotation.y, rotation.z);
    CreateKey(0.0);

    // Process chain.

    SelectItem(pointerItem.id);
    GoalItem(goalItem.id);
    FullTimeIK();
    if (mymatch)
        MatchGoalOrientation();

    nextItem = pointerItem.parent;

    while (!done)
    {

        if (nextItem.id == rootItem.id)
        {
            done = TRUE;
        }

        SelectItem(nextItem.id);
        if (myHtoggle and !myIKontop)
            HController(4);
        if (myPtoggle)
            PController(4);
        if (myBtoggle)
            BController(4);

        if (nextItem.parent)
            nextItem = nextItem.parent;
        else
            break;
    }

    if (myIKontop)
        HController(4);

    SelectItem(nextItem.id);
    if(nextItem.id != rootItem.id)
    {
        UnaffectedByIK();
    }
    //Cleanup

    SelectItem(goalItem.id);

//------------------------------------------------------------------------

    store("MGmymatch",mymatch);
    store("MGmyHtoggle",myHtoggle);
    store("MGmyPtoggle",myPtoggle);
    store("MGmyBtoggle",myBtoggle);
    store("MGmyIKontop",myIKontop);


}



